def K_xx(x_i, σ = .1) :
    return (-sqdistances(x_i,x_i)/σ**2 ).exp()

def H(q, p) :
    return .5 * scal( p, K_xx(q) @ p )

class PathRegistration(Model) :
    "Find the optimal mass positions."
    def __init__(self, α) :
        "Defines the parameters of a free deformation of α."
        super(Model, self).__init__()
        self.α, self.x = α[0].detach(), α[1].detach()
        self.p_ts = ParameterList([ 
            Parameter(torch.zeros_like(self.x))
            for i in range(10) ] )

    def __call__(self, t=1.) :
        "Applies the model on some point cloud, an N-by-2 tensor."
        x_t = self.x
        for p_t in self.p_ts :
            x_t = x_t + .1 * K_xx(x_t) @ (t*p_t)
        return self.α, x_t
    
    def cost(self, target) :
        "Returns a cost to optimize."
        data = fidelity( self(), target)
        
        reg_geom = torch.tensor(0.)
        x_t = self.x
        for p_t in self.p_ts :
            reg_geom = reg_geom + .1*H(x_t, p_t)
            x_t = x_t + .1 * K_xx(x_t) @ p_t
            
        return 1000*data + reg_geom

p_reg = train_and_display(PathRegistration, α, β)
