
#######################################################################################
#         Load, Sample, Display density from png source                               #
#######################################################################################

import numpy as np
import torch
from random import choices
from scipy import misc
from matplotlib import pyplot as plt


def load_image(fname) :
    img = misc.imread(fname, flatten = True) # Grayscale
    img = (img[::-1, :])  / 255.
    return 1 - img

def draw_samples(A, n) :
    xg, yg = np.meshgrid( np.linspace(0,1,A.shape[0]), np.linspace(0,1,A.shape[1]) )
    
    grid = list( zip(xg.ravel(), yg.ravel()) )
    dens = A.ravel() / A.sum()
    dots = np.array( choices(grid, dens, k=n ) )
    dots += (.5/A.shape[0]) * np.random.standard_normal(dots.shape)

    weights = torch.ones(n,1).type(dtype) / n
    return weights, torch.from_numpy(dots).type_as(A)

def extract_point_cloud(I) :
    # Threshold, to extract the relevant indices ---------------------------------------
    ind = (I > .001).nonzero()

    # Extract the weights --------------------------------------------------------------
    D = len(I.shape)
    if   D == 2 : α_i = I[ind[:,0], ind[:,1]]
    elif D == 3 : α_i = I[ind[:,0], ind[:,1], ind[:,2]]
    else : raise NotImplementedError()

    # Normalize weights and positions --------------------------------------------------
    x_i = ind.float() / I.shape[0]
    α_i = α_i / α_i.sum()

    return α_i.view(-1,1), x_i

def load_measure(fname, n=None, dtype=torch.FloatTensor) :
    img = dtype( load_image(fname) )
    if n is None :
        img = np.swapaxes(img, 0,1 )
        return extract_point_cloud(img)
    else :
        return draw_samples(img, n)
